searchData={"items":[{"type":"module","title":"SwarmEx","doc":"SwarmEx is an Elixir library for lightweight, controllable, and testable AI agent orchestration.\n\nIt provides a simple API for creating and managing networks of AI agents, with features including:\n\n- Agent lifecycle management\n- Message routing between agents\n- Network state management\n- Error handling and recovery","ref":"SwarmEx.html"},{"type":"module","title":"Example - SwarmEx","doc":"# Create a new agent network\n    {:ok, network} = SwarmEx.create_network()\n\n    # Define a tool as a regular module with functions\n    defmodule ClassifyTool do\n      def classify(text) do\n        # Perform classification\n        {:ok, result}\n      end\n    end\n\n    # Define an agent that uses the tool\n    defmodule MyAgent do\n      use SwarmEx.Agent\n\n      def init(opts), do: {:ok, opts}\n\n      def handle_message(msg, state) do\n        case ClassifyTool.classify(msg) do\n          {:ok, result} -> {:ok, result, state}\n          error -> error\n        end\n      end\n    end\n\n    # Add an agent to the network\n    {:ok, agent_pid} = SwarmEx.create_agent(network, MyAgent)\n\n    # Send a message to the agent\n    {:ok, response} = SwarmEx.send_message(agent_pid, \"Hello!\")","ref":"SwarmEx.html#module-example"},{"type":"function","title":"SwarmEx.create_agent/3","doc":"Creates a new agent in the given network.","ref":"SwarmEx.html#create_agent/3"},{"type":"function","title":"Options - SwarmEx.create_agent/3","doc":"* `:name` - Optional name for the agent\n  * `:instruction` - Base instruction/prompt for the agent\n  * All other options are passed to the agent's init/1 callback","ref":"SwarmEx.html#create_agent/3-options"},{"type":"function","title":"Examples - SwarmEx.create_agent/3","doc":"{:ok, agent} = SwarmEx.create_agent(network, MyAgent)\n    {:ok, agent} = SwarmEx.create_agent(network, MyAgent, name: \"processor\")","ref":"SwarmEx.html#create_agent/3-examples"},{"type":"function","title":"SwarmEx.create_network/1","doc":"Creates a new agent network with the given configuration.","ref":"SwarmEx.html#create_network/1"},{"type":"function","title":"Options - SwarmEx.create_network/1","doc":"* `:name` - Optional name for the network\n  * `:context` - Initial context map (default: %{})\n  * All other options are passed to the underlying Client","ref":"SwarmEx.html#create_network/1-options"},{"type":"function","title":"Examples - SwarmEx.create_network/1","doc":"{:ok, network} = SwarmEx.create_network()\n    {:ok, named_network} = SwarmEx.create_network(name: \"primary_network\")","ref":"SwarmEx.html#create_network/1-examples"},{"type":"function","title":"SwarmEx.list_agents/1","doc":"Lists all active agents in a network.","ref":"SwarmEx.html#list_agents/1"},{"type":"function","title":"Examples - SwarmEx.list_agents/1","doc":"{:ok, agent_ids} = SwarmEx.list_agents(network)","ref":"SwarmEx.html#list_agents/1-examples"},{"type":"function","title":"SwarmEx.register_tool/2","doc":"Registers a new tool that can be used by agents in the network.\n\nThis function is deprecated. Instead of using the Tool API, define your tools as regular modules\nwith functions. See the module documentation for examples.","ref":"SwarmEx.html#register_tool/2"},{"type":"function","title":"Examples - SwarmEx.register_tool/2","doc":"SwarmEx.register_tool(MyTool, max_retries: 3)","ref":"SwarmEx.html#register_tool/2-examples"},{"type":"function","title":"SwarmEx.send_message/3","doc":"Sends a message to an agent identified by ID within a network and waits for the response.","ref":"SwarmEx.html#send_message/3"},{"type":"function","title":"Examples - SwarmEx.send_message/3","doc":"{:ok, response} = SwarmEx.send_message(network, \"agent_id\", \"Process this\")","ref":"SwarmEx.html#send_message/3-examples"},{"type":"function","title":"SwarmEx.send_message_to_pid/2","doc":"Sends a message to an agent identified by PID and waits for the response.","ref":"SwarmEx.html#send_message_to_pid/2"},{"type":"function","title":"Examples - SwarmEx.send_message_to_pid/2","doc":"{:ok, response} = SwarmEx.send_message_to_pid(agent_pid, \"Process this\")","ref":"SwarmEx.html#send_message_to_pid/2-examples"},{"type":"function","title":"SwarmEx.stop_agent/2","doc":"Stops an agent and removes it from its network.","ref":"SwarmEx.html#stop_agent/2"},{"type":"function","title":"Examples - SwarmEx.stop_agent/2","doc":":ok = SwarmEx.stop_agent(agent)\n    :ok = SwarmEx.stop_agent(agent, :shutdown)","ref":"SwarmEx.html#stop_agent/2-examples"},{"type":"function","title":"SwarmEx.update_context/2","doc":"Updates the shared context for a network of agents.","ref":"SwarmEx.html#update_context/2"},{"type":"function","title":"Examples - SwarmEx.update_context/2","doc":":ok = SwarmEx.update_context(network, %{key: \"value\"})","ref":"SwarmEx.html#update_context/2-examples"},{"type":"function","title":"SwarmEx.version/0","doc":"Returns the version of the SwarmEx library.","ref":"SwarmEx.html#version/0"},{"type":"type","title":"SwarmEx.agent/0","doc":"","ref":"SwarmEx.html#t:agent/0"},{"type":"type","title":"SwarmEx.message/0","doc":"","ref":"SwarmEx.html#t:message/0"},{"type":"type","title":"SwarmEx.network/0","doc":"","ref":"SwarmEx.html#t:network/0"},{"type":"type","title":"SwarmEx.response/0","doc":"","ref":"SwarmEx.html#t:response/0"},{"type":"behaviour","title":"SwarmEx.Agent","doc":"Defines the behavior and implementation for SwarmEx agents.\n\nEach agent in the SwarmEx system is a process that can:\n- Process messages from other agents or clients\n- Maintain internal state\n- Participate in agent networks\n- Execute functions from tool modules\n- Perform self-health checks and recovery","ref":"SwarmEx.Agent.html"},{"type":"function","title":"SwarmEx.Agent.create/2","doc":"Creates a new agent process with the given module and options.","ref":"SwarmEx.Agent.html#create/2"},{"type":"callback","title":"SwarmEx.Agent.handle_handoff/2","doc":"","ref":"SwarmEx.Agent.html#c:handle_handoff/2"},{"type":"callback","title":"SwarmEx.Agent.handle_message/2","doc":"","ref":"SwarmEx.Agent.html#c:handle_message/2"},{"type":"callback","title":"SwarmEx.Agent.handle_recovery/2","doc":"","ref":"SwarmEx.Agent.html#c:handle_recovery/2"},{"type":"callback","title":"SwarmEx.Agent.health_check/1","doc":"","ref":"SwarmEx.Agent.html#c:health_check/1"},{"type":"function","title":"SwarmEx.Agent.stop/2","doc":"Stops an agent process.","ref":"SwarmEx.Agent.html#stop/2"},{"type":"function","title":"SwarmEx.Agent.validate_agent/1","doc":"Validates the agent implementation to ensure all required callbacks are implemented correctly.","ref":"SwarmEx.Agent.html#validate_agent/1"},{"type":"type","title":"SwarmEx.Agent.error/0","doc":"","ref":"SwarmEx.Agent.html#t:error/0"},{"type":"type","title":"SwarmEx.Agent.health_status/0","doc":"","ref":"SwarmEx.Agent.html#t:health_status/0"},{"type":"type","title":"SwarmEx.Agent.message/0","doc":"","ref":"SwarmEx.Agent.html#t:message/0"},{"type":"type","title":"SwarmEx.Agent.response/0","doc":"","ref":"SwarmEx.Agent.html#t:response/0"},{"type":"type","title":"SwarmEx.Agent.state/0","doc":"","ref":"SwarmEx.Agent.html#t:state/0"},{"type":"module","title":"SwarmEx.Application","doc":"The main SwarmEx application supervisor.\nResponsible for starting and supervising core system components.","ref":"SwarmEx.Application.html"},{"type":"module","title":"SwarmEx.Client","doc":"Main supervisor for agent networks. Manages agent lifecycles,\nmessage passing, error recovery and context variables.\n\nThe Client acts as a coordinator for a network of AI agents, handling:\n- Agent lifecycle management (creation, termination)\n- Message routing between agents\n- Network state and context maintenance\n- Error handling and recovery","ref":"SwarmEx.Client.html"},{"type":"function","title":"SwarmEx.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SwarmEx.Client.html#child_spec/1"},{"type":"function","title":"SwarmEx.Client.create_agent/3","doc":"Creates a new agent in the network.","ref":"SwarmEx.Client.html#create_agent/3"},{"type":"function","title":"Options - SwarmEx.Client.create_agent/3","doc":"All options are passed to the agent's init function.","ref":"SwarmEx.Client.html#create_agent/3-options"},{"type":"function","title":"SwarmEx.Client.list_agents/1","doc":"Lists all active agents in the network.","ref":"SwarmEx.Client.html#list_agents/1"},{"type":"function","title":"SwarmEx.Client.send_message/3","doc":"Sends a message to a specific agent in the network.","ref":"SwarmEx.Client.html#send_message/3"},{"type":"function","title":"SwarmEx.Client.start_link/1","doc":"Starts a new agent network with the given options.","ref":"SwarmEx.Client.html#start_link/1"},{"type":"function","title":"Options - SwarmEx.Client.start_link/1","doc":"* `:network_id` - Custom identifier for the network (optional)\n  * `:context` - Initial context map (default: %{})\n  * `:registry` - Custom registry for agent processes (optional)","ref":"SwarmEx.Client.html#start_link/1-options"},{"type":"function","title":"SwarmEx.Client.update_context/2","doc":"Updates the network context with new values.","ref":"SwarmEx.Client.html#update_context/2"},{"type":"type","title":"SwarmEx.Client.t/0","doc":"","ref":"SwarmEx.Client.html#t:t/0"},{"type":"module","title":"SwarmEx.ClientSupervisor","doc":"Supervisor for SwarmEx client processes.\n\nManages the lifecycle of client processes with proper error handling and restart strategies.\nEach client represents a network of agents and is supervised independently.","ref":"SwarmEx.ClientSupervisor.html"},{"type":"function","title":"SwarmEx.ClientSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SwarmEx.ClientSupervisor.html#child_spec/1"},{"type":"function","title":"SwarmEx.ClientSupervisor.count_clients/0","doc":"Counts the number of active client processes.","ref":"SwarmEx.ClientSupervisor.html#count_clients/0"},{"type":"function","title":"SwarmEx.ClientSupervisor.list_clients/0","doc":"Lists all active client processes.","ref":"SwarmEx.ClientSupervisor.html#list_clients/0"},{"type":"function","title":"SwarmEx.ClientSupervisor.start_client/1","doc":"Starts a new client process under supervision.","ref":"SwarmEx.ClientSupervisor.html#start_client/1"},{"type":"function","title":"SwarmEx.ClientSupervisor.start_link/1","doc":"","ref":"SwarmEx.ClientSupervisor.html#start_link/1"},{"type":"function","title":"SwarmEx.ClientSupervisor.terminate_client/1","doc":"Terminates a client process and all its associated agents.","ref":"SwarmEx.ClientSupervisor.html#terminate_client/1"},{"type":"module","title":"SwarmEx.Error","doc":"Defines error types for the SwarmEx library.","ref":"SwarmEx.Error.html"},{"type":"module","title":"SwarmEx.Telemetry","doc":"Telemetry integration for SwarmEx.\nProvides comprehensive metrics and event tracking for agent activities.","ref":"SwarmEx.Telemetry.html"},{"type":"module","title":"Event Categories - SwarmEx.Telemetry","doc":"","ref":"SwarmEx.Telemetry.html#module-event-categories"},{"type":"module","title":"Agent Lifecycle Events - SwarmEx.Telemetry","doc":"* `[:swarm_ex, :agent, :init]` - Agent initialization\n  * Measurement: `:system_time`\n  * Metadata: `:agent_id`, `:network_id`, `:correlation_id`\n\n* `[:swarm_ex, :agent, :terminate]` - Agent termination\n  * Measurement: `:system_time`, `:uptime`\n  * Metadata: `:agent_id`, `:network_id`, `:correlation_id`, `:reason`","ref":"SwarmEx.Telemetry.html#module-agent-lifecycle-events"},{"type":"module","title":"Message Processing Events - SwarmEx.Telemetry","doc":"* `[:swarm_ex, :agent, :message, :start]` - Message processing start\n  * Measurement: `:system_time`\n  * Metadata: `:agent_id`, `:message_type`, `:network_id`, `:correlation_id`, `:message_size`\n\n* `[:swarm_ex, :agent, :message, :stop]` - Message processing completion\n  * Measurement: `:duration` (microseconds), `:queue_time` (milliseconds)\n  * Metadata: `:agent_id`, `:message_type`, `:network_id`, `:correlation_id`, `:result`","ref":"SwarmEx.Telemetry.html#module-message-processing-events"},{"type":"module","title":"Tool Execution Events - SwarmEx.Telemetry","doc":"* `[:swarm_ex, :tool, :execute, :start]` - Tool execution start\n  * Measurement: `:system_time`\n  * Metadata: `:tool_name`, `:agent_id`, `:network_id`, `:correlation_id`, `:args`\n\n* `[:swarm_ex, :tool, :execute, :stop]` - Tool execution completion\n  * Measurement: `:duration` (microseconds)\n  * Metadata: `:tool_name`, `:agent_id`, `:network_id`, `:correlation_id`, `:result`","ref":"SwarmEx.Telemetry.html#module-tool-execution-events"},{"type":"module","title":"Health and Resource Events - SwarmEx.Telemetry","doc":"* `[:swarm_ex, :health, :check]` - Periodic health check\n  * Measurement: `:memory` (bytes), `:process_count`, `:message_queue_length`\n  * Metadata: `:node`, `:network_id`\n\n* `[:swarm_ex, :agent, :memory]` - Agent memory usage\n  * Measurement: `:memory` (bytes), `:heap_size` (bytes), `:stack_size` (bytes)\n  * Metadata: `:agent_id`, `:network_id`","ref":"SwarmEx.Telemetry.html#module-health-and-resource-events"},{"type":"function","title":"SwarmEx.Telemetry.attach/1","doc":"Attaches telemetry event handlers and starts periodic health checks.\nCall this when your application starts.","ref":"SwarmEx.Telemetry.html#attach/1"},{"type":"function","title":"Options - SwarmEx.Telemetry.attach/1","doc":"* `:health_check_interval` - Interval in milliseconds between health checks. Defaults to 60_000 (1 minute)","ref":"SwarmEx.Telemetry.html#attach/1-options"},{"type":"function","title":"SwarmEx.Telemetry.generate_correlation_id/0","doc":"Generates a new correlation ID for request tracing.","ref":"SwarmEx.Telemetry.html#generate_correlation_id/0"},{"type":"function","title":"SwarmEx.Telemetry.handle_info/2","doc":"","ref":"SwarmEx.Telemetry.html#handle_info/2"},{"type":"function","title":"SwarmEx.Telemetry.report_agent_memory/2","doc":"Emits agent memory usage metrics.","ref":"SwarmEx.Telemetry.html#report_agent_memory/2"},{"type":"function","title":"SwarmEx.Telemetry.span_agent_message/4","doc":"Emits an agent message event with timing and tracing information.","ref":"SwarmEx.Telemetry.html#span_agent_message/4"},{"type":"function","title":"SwarmEx.Telemetry.span_tool_execution/5","doc":"Emits a tool execution event with timing and tracing information.","ref":"SwarmEx.Telemetry.html#span_tool_execution/5"},{"type":"type","title":"SwarmEx.Telemetry.base_metadata/0","doc":"Metadata common to all events","ref":"SwarmEx.Telemetry.html#t:base_metadata/0"},{"type":"type","title":"SwarmEx.Telemetry.correlation_id/0","doc":"Correlation ID for request tracing","ref":"SwarmEx.Telemetry.html#t:correlation_id/0"},{"type":"behaviour","title":"SwarmEx.Tool","doc":"DEPRECATED: This module is deprecated. Tools should be implemented as regular modules with functions instead.","ref":"SwarmEx.Tool.html"},{"type":"behaviour","title":"Migration Guide - SwarmEx.Tool","doc":"Instead of using the Tool behavior, define your tools as regular modules with functions:","ref":"SwarmEx.Tool.html#module-migration-guide"},{"type":"behaviour","title":"Before: - SwarmEx.Tool","doc":"defmodule MyTool do\n      @behaviour SwarmEx.Tool\n\n      @impl true\n      def execute(args) do\n        # Perform tool operation\n        {:ok, result}\n      end\n\n      @impl true\n      def validate(args) do\n        # Validate incoming arguments\n        :ok\n      end\n\n      @impl true\n      def cleanup(args) do\n        # Cleanup any resources\n        :ok\n      end\n    end","ref":"SwarmEx.Tool.html#module-before"},{"type":"behaviour","title":"After: - SwarmEx.Tool","doc":"defmodule MyTool do\n      def process(args) do\n        # Validate args if needed\n        with :ok <- validate_args(args),\n             {:ok, result} <- do_process(args) do\n          cleanup(args)\n          {:ok, result}\n        end\n      end\n\n      defp validate_args(args) do\n        # Optional validation\n        :ok\n      end\n\n      defp do_process(args) do\n        # Main processing logic\n        {:ok, result}\n      end\n\n      defp cleanup(args) do\n        # Optional cleanup\n        :ok\n      end\n    end\n\nThe new approach:\n- Is simpler and more idiomatic Elixir\n- Gives more flexibility in function naming and implementation\n- Allows for better integration with other libraries\n- Reduces boilerplate code\n- Makes testing easier","ref":"SwarmEx.Tool.html#module-after"},{"type":"callback","title":"SwarmEx.Tool.cleanup/1","doc":"","ref":"SwarmEx.Tool.html#c:cleanup/1"},{"type":"callback","title":"SwarmEx.Tool.execute/1","doc":"","ref":"SwarmEx.Tool.html#c:execute/1"},{"type":"function","title":"SwarmEx.Tool.execute/3","doc":"DEPRECATED: Use direct function calls instead.\n\nSafely executes a tool with the given arguments and options.","ref":"SwarmEx.Tool.html#execute/3"},{"type":"function","title":"Options - SwarmEx.Tool.execute/3","doc":"* `:timeout` - Maximum time in milliseconds to wait (default: 5000)\n  * `:retries` - Number of retry attempts (default: 3)\n  * `:validate_args` - Whether to validate arguments (default: true)","ref":"SwarmEx.Tool.html#execute/3-options"},{"type":"function","title":"SwarmEx.Tool.get_config/1","doc":"DEPRECATED: Use regular module configuration instead.\n\nRetrieves the configuration for a registered tool.","ref":"SwarmEx.Tool.html#get_config/1"},{"type":"function","title":"SwarmEx.Tool.register/2","doc":"DEPRECATED: Use regular module configuration instead.\n\nRegisters a tool configuration in the runtime.","ref":"SwarmEx.Tool.html#register/2"},{"type":"callback","title":"SwarmEx.Tool.validate/1","doc":"","ref":"SwarmEx.Tool.html#c:validate/1"},{"type":"function","title":"SwarmEx.Tool.validate_tool/1","doc":"DEPRECATED: Use regular module definition instead.\n\nValidates that a module implements the Tool behavior correctly.","ref":"SwarmEx.Tool.html#validate_tool/1"},{"type":"type","title":"SwarmEx.Tool.args/0","doc":"","ref":"SwarmEx.Tool.html#t:args/0"},{"type":"type","title":"SwarmEx.Tool.cleanup_result/0","doc":"","ref":"SwarmEx.Tool.html#t:cleanup_result/0"},{"type":"type","title":"SwarmEx.Tool.options/0","doc":"","ref":"SwarmEx.Tool.html#t:options/0"},{"type":"type","title":"SwarmEx.Tool.result/0","doc":"","ref":"SwarmEx.Tool.html#t:result/0"},{"type":"type","title":"SwarmEx.Tool.t/0","doc":"","ref":"SwarmEx.Tool.html#t:t/0"},{"type":"type","title":"SwarmEx.Tool.validation_result/0","doc":"","ref":"SwarmEx.Tool.html#t:validation_result/0"},{"type":"module","title":"SwarmEx.Tools.CodeInterpreter","doc":"DEPRECATED: This module is deprecated. Use a regular module with functions instead.","ref":"SwarmEx.Tools.CodeInterpreter.html"},{"type":"module","title":"Migration Guide - SwarmEx.Tools.CodeInterpreter","doc":"Instead of using the Tool behavior, implement code interpretation as regular functions:\n\n```elixir\ndefmodule MyCodeInterpreter do\n  def execute_code(language, code, opts \\ []) do\n    case language do\n      \"elixir\" -> execute_elixir(code, opts)\n      \"python\" -> execute_python(code, opts)\n      _ -> {:error, :unsupported_language}\n    end\n  end\n\n  defp execute_elixir(code, opts) do\n    # Implement sandboxed Elixir code execution\n    {:error, :not_implemented}\n  end\n\n  defp execute_python(code, opts) do\n    # Implement Python code execution via port\n    {:error, :not_implemented}\n  end\n\n  defp validate_code(language, code) do\n    # Implement validation logic\n    :ok\n  end\nend\n```\n\nThen use it directly in your agent:\n\n```elixir\ndefmodule MyAgent do\n  use SwarmEx.Agent\n\n  def handle_message(%{language: lang, code: code} = msg, state) do\n    case MyCodeInterpreter.execute_code(lang, code) do\n      {:ok, result} -> {:ok, result, state}\n      error -> error\n    end\n  end\nend\n```","ref":"SwarmEx.Tools.CodeInterpreter.html#module-migration-guide"},{"type":"type","title":"SwarmEx.Tools.CodeInterpreter.code_block/0","doc":"","ref":"SwarmEx.Tools.CodeInterpreter.html#t:code_block/0"},{"type":"module","title":"SwarmEx.Tools.Retrieval","doc":"DEPRECATED: This module is deprecated. Use a regular module with functions instead.","ref":"SwarmEx.Tools.Retrieval.html"},{"type":"module","title":"Migration Guide - SwarmEx.Tools.Retrieval","doc":"Instead of using the Tool behavior, implement retrieval as regular functions:\n\n```elixir\ndefmodule MyRetrieval do\n  @type query :: %{\n          query: String.t(),\n          data_source: atom(),\n          limit: integer(),\n          filters: map()\n        }\n\n  @type retrieval_result :: %{\n          content: String.t(),\n          source: String.t(),\n          relevance_score: float(),\n          metadata: map()\n        }\n\n  def search(query, data_source, opts \\ []) do\n    with :ok <- validate_query(query),\n         :ok <- validate_data_source(data_source) do\n      do_search(query, data_source, opts)\n    end\n  end\n\n  def search_vector_store(query, opts \\ []) do\n    # Implement vector store search\n    {:error, :not_implemented}\n  end\n\n  def search_document_store(query, opts \\ []) do\n    # Implement document store search\n    {:error, :not_implemented}\n  end\n\n  def search_knowledge_base(query, opts \\ []) do\n    # Implement knowledge base search\n    {:error, :not_implemented}\n  end\n\n  defp do_search(query, data_source, opts) do\n    case data_source do\n      :vector_store -> search_vector_store(query, opts)\n      :document_store -> search_document_store(query, opts)\n      :knowledge_base -> search_knowledge_base(query, opts)\n      _ -> {:error, :unsupported_data_source}\n    end\n  end\n\n  defp validate_query(query) when is_binary(query), do: :ok\n  defp validate_query(_), do: {:error, :invalid_query}\n\n  defp validate_data_source(source)\n       when source in [:vector_store, :document_store, :knowledge_base],\n       do: :ok\n  defp validate_data_source(_), do: {:error, :invalid_data_source}\nend\n```\n\nThen use it directly in your agent:\n\n```elixir\ndefmodule MyAgent do\n  use SwarmEx.Agent\n\n  def handle_message(%{query: query, source: source} = msg, state) do\n    case MyRetrieval.search(query, source) do\n      {:ok, results} -> {:ok, results, state}\n      error -> error\n    end\n  end\nend\n```","ref":"SwarmEx.Tools.Retrieval.html#module-migration-guide"},{"type":"type","title":"SwarmEx.Tools.Retrieval.query/0","doc":"","ref":"SwarmEx.Tools.Retrieval.html#t:query/0"},{"type":"type","title":"SwarmEx.Tools.Retrieval.retrieval_result/0","doc":"","ref":"SwarmEx.Tools.Retrieval.html#t:retrieval_result/0"},{"type":"module","title":"SwarmEx.Utils","doc":"Utility functions for SwarmEx operations.\nProvides helper functions for common operations across the library.","ref":"SwarmEx.Utils.html"},{"type":"function","title":"SwarmEx.Utils.deep_merge/2","doc":"Deep merges two maps recursively. If the same key exists in both maps\nand both values are maps, they are merged recursively. Otherwise, the\nvalue from the second map takes precedence.","ref":"SwarmEx.Utils.html#deep_merge/2"},{"type":"function","title":"Examples - SwarmEx.Utils.deep_merge/2","doc":"iex> deep_merge(%{a: 1, b: %{c: 2}}, %{b: %{d: 3}, e: 4})\n    %{a: 1, b: %{c: 2, d: 3}, e: 4}\n\n    iex> deep_merge(%{a: %{b: 1}}, %{a: 2})\n    %{a: 2}","ref":"SwarmEx.Utils.html#deep_merge/2-examples"},{"type":"function","title":"SwarmEx.Utils.deserialize/1","doc":"Deserializes a string back into a term.","ref":"SwarmEx.Utils.html#deserialize/1"},{"type":"function","title":"Examples - SwarmEx.Utils.deserialize/1","doc":"iex> deserialize(\"{\":name\":\"John\",\":age\":30}\")\n    {:ok, %{name: \"John\", age: 30}}\n\n    iex> deserialize(\"invalid json\")\n    {:error, :invalid_json}","ref":"SwarmEx.Utils.html#deserialize/1-examples"},{"type":"function","title":"SwarmEx.Utils.format_error/1","doc":"Formats an error tuple into a standardized error structure.","ref":"SwarmEx.Utils.html#format_error/1"},{"type":"function","title":"Examples - SwarmEx.Utils.format_error/1","doc":"iex> format_error({:error, \"Something went wrong\"})\n    %SwarmEx.Error.AgentError{message: \"Something went wrong\", reason: :unknown}\n\n    iex> format_error(%RuntimeError{message: \"Oops\"})\n    %SwarmEx.Error.AgentError{message: \"Oops\", reason: :runtime_error}","ref":"SwarmEx.Utils.html#format_error/1-examples"},{"type":"function","title":"SwarmEx.Utils.generate_id/1","doc":"Generates a unique identifier with an optional prefix.","ref":"SwarmEx.Utils.html#generate_id/1"},{"type":"function","title":"Examples - SwarmEx.Utils.generate_id/1","doc":"iex> generate_id()\n    \"c1dd6960-7b91-4ca1-b853-c01c7f24d1aa\"\n\n    iex> generate_id(\"user\")\n    \"user_c1dd6960-7b91-4ca1-b853-c01c7f24d1aa\"","ref":"SwarmEx.Utils.html#generate_id/1-examples"},{"type":"function","title":"SwarmEx.Utils.log/3","doc":"Logs a message with additional context and metadata.\nEnsures consistent log formatting across the application.","ref":"SwarmEx.Utils.html#log/3"},{"type":"function","title":"Examples - SwarmEx.Utils.log/3","doc":"iex> log(:info, \"Processing message\", agent_id: \"123\")\n    :ok\n\n    iex> log(:error, \"Failed to process\", [error: \"timeout\", agent_id: \"123\"])\n    :ok","ref":"SwarmEx.Utils.html#log/3-examples"},{"type":"function","title":"SwarmEx.Utils.safely_execute/2","doc":"Safely executes a function with timeout and retry logic.","ref":"SwarmEx.Utils.html#safely_execute/2"},{"type":"function","title":"Options - SwarmEx.Utils.safely_execute/2","doc":"* `:timeout` - Maximum time in milliseconds to wait for each attempt (default: 5000)\n  * `:retries` - Number of retry attempts (default: 3)\n  * `:backoff_base` - Base time in milliseconds for backoff calculation (default: 100)\n  * `:backoff_type` - Type of backoff, either :exponential or :linear (default: :exponential)","ref":"SwarmEx.Utils.html#safely_execute/2-options"},{"type":"function","title":"Examples - SwarmEx.Utils.safely_execute/2","doc":"iex> safely_execute(fn -> {:ok, :success} end)\n    {:ok, :success}\n\n    iex> safely_execute(fn -> raise \"error\" end)\n    {:error, %RuntimeError{message: \"error\"}}","ref":"SwarmEx.Utils.html#safely_execute/2-examples"},{"type":"function","title":"SwarmEx.Utils.serialize/1","doc":"Serializes a term into a string representation.","ref":"SwarmEx.Utils.html#serialize/1"},{"type":"function","title":"Examples - SwarmEx.Utils.serialize/1","doc":"iex> serialize(%{name: \"John\", age: 30})\n    {:ok, \"{\":name\":\"John\",\":age\":30}\"}\n\n    iex> serialize(%{invalid: make_ref()})\n    {:error, :unserializable_term}","ref":"SwarmEx.Utils.html#serialize/1-examples"},{"type":"function","title":"SwarmEx.Utils.validate_schema/2","doc":"Validates a map against a schema of required and optional keys.","ref":"SwarmEx.Utils.html#validate_schema/2"},{"type":"function","title":"Examples - SwarmEx.Utils.validate_schema/2","doc":"iex> schema = [required: [:name, :age], optional: [:email]]\n    iex> validate_schema(%{name: \"John\", age: 30}, schema)\n    :ok\n\n    iex> validate_schema(%{name: \"John\"}, [required: [:name, :age]])\n    {:error, {:missing_required_keys, [:age]}}","ref":"SwarmEx.Utils.html#validate_schema/2-examples"},{"type":"type","title":"SwarmEx.Utils.retry_opts/0","doc":"","ref":"SwarmEx.Utils.html#t:retry_opts/0"},{"type":"exception","title":"SwarmEx.Error.AgentError","doc":"Raised when an agent encounters an error","ref":"SwarmEx.Error.AgentError.html"},{"type":"exception","title":"SwarmEx.Error.ClientError","doc":"Raised when a client operation fails","ref":"SwarmEx.Error.ClientError.html"},{"type":"exception","title":"SwarmEx.Error.NetworkError","doc":"Raised when a network operation fails","ref":"SwarmEx.Error.NetworkError.html"},{"type":"exception","title":"SwarmEx.Error.ToolError","doc":"Raised when a tool operation fails","ref":"SwarmEx.Error.ToolError.html"},{"type":"exception","title":"SwarmEx.Error.ValidationError","doc":"Raised when validation fails","ref":"SwarmEx.Error.ValidationError.html"},{"type":"extras","title":"SwarmEx  🐝 🐝 🐝","doc":"# SwarmEx  🐝 🐝 🐝\n\n[![Hex Version](https://img.shields.io/hexpm/v/swarm_ex.svg)](https://hex.pm/packages/swarm_ex)\n\nSwarmEx is an Elixir library for lightweight, controllable, and testable AI agent orchestration. It provides primitives for creating and coordinating networks of AI agents, leveraging Elixir's native strengths in concurrency and fault tolerance.","ref":"readme.html"},{"type":"extras","title":"Installation - SwarmEx  🐝 🐝 🐝","doc":"The package can be installed by adding `swarm_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:swarm_ex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Features - SwarmEx  🐝 🐝 🐝","doc":"- Lightweight agent orchestration\n- Tool integration framework\n- Built-in telemetry and observability\n- Robust error handling\n- Clear developer experience","ref":"readme.html#features"},{"type":"extras","title":"Quick Start - SwarmEx  🐝 🐝 🐝","doc":"[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2Fnrrso%2Fswarm_ex%2Frefs%2Fheads%2Fmain%2Flivebooks%2Fswarm_ex.livemd)","ref":"readme.html#quick-start"},{"type":"extras","title":"Documentation - SwarmEx  🐝 🐝 🐝","doc":"The docs can be found at [https://hexdocs.pm/swarm_ex](https://hexdocs.pm/swarm_ex).","ref":"readme.html#documentation"},{"type":"extras","title":"Testing - SwarmEx  🐝 🐝 🐝","doc":"```bash\nmix test\n```","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - SwarmEx  🐝 🐝 🐝","doc":"1. Fork it\n2. Create your feature branch (`git checkout -b feature/my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin feature/my-new-feature`)\n5. Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - SwarmEx  🐝 🐝 🐝","doc":"Apache License 2.0 - See LICENSE for details.","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}